/* mcd.c

   Calcola il massimo comun divisore di due numeri interi a e b
   trasmessi con la linea di comando

   Lo fa applicando la definizione in modo esaustivo, cio scorrendo tutti
   i numeri da min(a,b) a 2 e verificando se sono divisori di a e b.
   Per efficienza, testa a parte il caso min(a,b) e poi comincia da min(a,b)/2.
   Si ferma al primo divisore comune.

*/

#include <stdio.h>
#include <stdlib.h>

#include <time.h>


#define EXIT_COMMANDLINE 1

typedef enum {FALSE,TRUE} boolean;


void InterpretaLineaComando (int argc, char *argv[], long *pa, long *pb);

void StampaIstruzioni (char *comando);

long MassimoComunDivisore (long a, long b);


int main (int argc, char *argv[])
{
  long a, b;
  int MCD;
  time_t inizio, fine;
  double durata;


  /* Interpreta la linea di comando per caricare i due numeri */
  InterpretaLineaComando(argc,argv,&a,&b);

  /* Calcola il MCD */
  inizio = clock();
  MCD = MassimoComunDivisore(a,b);
  fine = clock();
  durata = (fine - inizio)/CLOCKS_PER_SEC;

  /* Stampa il MCD */
  printf("MCD: %ld\n",MCD);
  printf("Tempo: %.3lf secondi\n",durata);

  return EXIT_SUCCESS;
}


/* Interpreta la linea di comando per caricare i due numeri */
void InterpretaLineaComando (int argc, char *argv[], long *pa, long *pb)
{
  if (argc != 3)
  {
    fprintf(stderr,"Linea di comando errata!\n");
    StampaIstruzioni(argv[0]);
    exit(EXIT_COMMANDLINE);
  }
  else if ( (sscanf(argv[1],"%ld",pa) != 1) || (sscanf(argv[2],"%ld",pb) != 1) )
  {
    fprintf(stderr,"Linea di comando errata!\n");
    StampaIstruzioni(argv[0]);
    exit(EXIT_COMMANDLINE);
  }
  else if ( (*pa < 0) || (*pb < 0) )
  {
    fprintf(stderr,"Linea di comando errata!\n");
    StampaIstruzioni(argv[0]);
    exit(EXIT_COMMANDLINE);
  }
}


void StampaIstruzioni (char *comando)
{
  fprintf(stderr,"%s a b\n",comando);
  fprintf(stderr,"a: primo   termine (intero positivo)\n");
  fprintf(stderr,"b: secondo termine (intero positivo)\n");
}


long MassimoComunDivisore (long a, long b)
{
  long MCD, min;

  min = ( (a < b) ? a : b);
  if ( (a % min == 0) && (b % min == 0) ) return min;

  for (MCD = min/2; MCD > 1; MCD--)
    if ( (a % MCD == 0) && (b % MCD == 0) )
      break;

  return MCD;
}
